set echo on
set feedback on
set serveroutput on
var start_date  varchar2(80);
var end_date    varchar2(80);
var sid         varchar2(10);
var host        varchar2(30);
var user        varchar2(30);


spool 2.7.2_Update_Script.log;

begin
  select to_char(sysdate, 'mm-dd-yyyy hh24:mi:ss') into :start_date from dual;
  select instance_name into :sid from v$instance;
  select host_name into :host from v$instance;
  select user into :user from dual;

  dbms_output.put_line('Script executed on SID:  '||:sid);
  dbms_output.put_line('Script executed on host: '||:host);
  dbms_output.put_line('Script start date/time:  '||:start_date);
  dbms_output.put_line('Script executed by:      '||:user);
end;
/
SET DEFINE OFF;

CREATE TABLE VAP.PATIENT_CONFIDENTIALITY
(
  PATIENT_CONFIDENTIALITY_ID  INTEGER           NOT NULL,
  PATIENT_ICN                 VARCHAR2(255 BYTE) NOT NULL,
  CONFIDENTIALITY_CODE        VARCHAR2(255 BYTE) NOT NULL,
  STORED_TS                   DATE
)
TABLESPACE VAP_DATA01;


CREATE UNIQUE INDEX VAP.PK_PATIENT_CONFIDENTIALITY ON VAP.PATIENT_CONFIDENTIALITY
(PATIENT_CONFIDENTIALITY_ID)
TABLESPACE VAP_INDEX01;

CREATE UNIQUE INDEX VAP.UK_PAT_CONF_PATIENT_ICN ON VAP.PATIENT_CONFIDENTIALITY
(PATIENT_ICN)
TABLESPACE VAP_INDEX01;

CREATE SYNONYM VAP_APP.PATIENT_CONFIDENTIALITY FOR VAP.PATIENT_CONFIDENTIALITY;


ALTER TABLE VAP.PATIENT_CONFIDENTIALITY ADD (
  CONSTRAINT PK_PATIENT_CONFIDENTIALITY
  PRIMARY KEY
  (PATIENT_CONFIDENTIALITY_ID)
  USING INDEX VAP.PK_PATIENT_CONFIDENTIALITY
  ENABLE VALIDATE);

ALTER TABLE VAP.PATIENT_CONFIDENTIALITY ADD (
  CONSTRAINT UK_PAT_CONF_PATIENT_ICN
  UNIQUE (PATIENT_ICN)
  USING INDEX VAP.UK_PAT_CONF_PATIENT_ICN
  ENABLE VALIDATE);

GRANT SELECT ON VAP.PATIENT_CONFIDENTIALITY TO VAP_READ;

GRANT DELETE, INSERT, SELECT, UPDATE ON VAP.PATIENT_CONFIDENTIALITY TO VAP_USER;

INSERT INTO VAP.CONSENT_TYPE(CONSENT_TYPE_ID, NAME)
VALUES(110, 'Withhold for Treatment Authorization');

INSERT INTO VAP.CONSENT_TYPE(CONSENT_TYPE_ID, NAME)
VALUES(111, 'Withhold for Treatment Revocation');

--------------------------------------------------------
--  UPDATE DESCRIPTORS
--  These statements came from VAP-E team as part of Section 508 fixes
--  they made post- VAP 2.7.1. Manually "merged" them into this
--  SQL file for the VAP 2.7.2 release (eHX-E Build 3).
--------------------------------------------------------
UPDATE VAP.DESCRIPTOR SET TEXT='Add all organizations to the announcement list' WHERE NAME='batchAnnounceMoveAll';
UPDATE VAP.DESCRIPTOR SET TEXT='Add selected organizations to the announcement list' WHERE NAME='batchAnnounceMoveSelectedRight';
UPDATE VAP.DESCRIPTOR SET TEXT='Remove selected organizations from the announcement list' WHERE NAME='batchAnnounceMoveSelectedLeft';

commit;

Set Define on;

Begin
  select to_char(sysdate, 'mm-dd-yyyy hh24:mi:ss') into :end_date from dual;
  dbms_output.put_line('Script end date/time:    '||:end_date);
  dbms_output.put_line('Script executed in '||(to_date(:end_date, 'mm_dd_yyyy hh24:mi:ss') - to_date(:start_date, 'mm-dd-yyyy hh24:mi:ss')) *24*60*60||' seconds');
end;
/

Spool off;
