/**
 * DqTransmissionLog service utility class to avoid duplicating code.
 * 
 */
package gov.va.nvap.web.helper.report;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.json.JsonSanitizer;

import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.service.dataquality.DqTransmissionLog;

import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 *
 * 
 */
public class DqTransmissionLogReportHelper {
    
    static private final Log LOG = LogFactory.getLog(DqTransmissionLogReportHelper.class);
    
    /**
     * Transforms REST input JSON string into special response Object.
     * 
     * @param inputJsonStr
     * @return 
     */    
    public DqTransmissionLogReportHelper.QueryResponseObj transformRespJsonStrToQueryResponseObj(String inputJsonStr) {
        // Transform from JSON String to Object
        Gson gson = new GsonBuilder().create();
        DqTransmissionLogReportHelper.QueryResponseObj respObj = gson.fromJson(JsonSanitizer.sanitize(inputJsonStr), DqTransmissionLogReportHelper.QueryResponseObj.class);
        //System.out.println("respObj: toString "+respObj);
        LOG.debug("Returning QueryResponseObj toString: "+respObj);
        return respObj;
    }
    
    /**
     * Transforms response Object to Object with List<List> data, etc., 
     * and then to JSON string, in JSON format required by JQuery DataTables library, 
     * which is used in report AJAX processing to display report results in .jsp output.
     * 
     * @param respObj
     * @return 
     */
    public String transformJsonRespObjToDataTablesJsonStr(DqTransmissionLogReportHelper.QueryResponseObj respObj) { 
        String transformedJsonString = null;

        // Transform from Object to Object        
        DqTransmissionLogReportHelper.DqTransmissionLogDataTablesReport report = null;
        if(!NullChecker.isNullOrEmpty(respObj)) {
            report = new DqTransmissionLogReportHelper.DqTransmissionLogDataTablesReport();
            report.setTableName(respObj.getTableName());
            report.setRecordsFiltered(respObj.getResultCount());
            report.setRecordsTotal(new Integer(respObj.getResultCount()).toString());
            List<List<String>> dataList = new ArrayList<>();
            List<DqTransmissionLog> respDataList = respObj.data;
            for(DqTransmissionLog dqtl: respDataList) {
                List<String> curPropertyValueList = new ArrayList<>();
                // insert all values only of current dqtl in a list
                Long dqTransmissionId = dqtl.getDqTransmissionId();
                if(NullChecker.isNullOrEmpty(dqTransmissionId)) {
                    curPropertyValueList.add("");
                } else {
                    curPropertyValueList.add(new Long(dqTransmissionId).toString());
                }
                String batchId = dqtl.getBatchId();
                if(NullChecker.isNullOrEmpty(batchId) || batchId.equalsIgnoreCase("null")) {
                    curPropertyValueList.add("");
                } else {
                    curPropertyValueList.add(batchId);
                }
                String eventType = dqtl.getEventType();
                if(NullChecker.isNullOrEmpty(eventType) || eventType.equalsIgnoreCase("null")) {
                    curPropertyValueList.add("");
                } else {
                    curPropertyValueList.add(eventType);
                }
                String startTime = dqtl.getStartTime();
                if(NullChecker.isNullOrEmpty(startTime)) {
                    curPropertyValueList.add("");
                } else {
                    String startTimeDateStr = this.convertTimestampStrToDateStr(startTime);
                    curPropertyValueList.add(startTimeDateStr);
                }
                String endTime = dqtl.getEndTime();
                if(NullChecker.isNullOrEmpty(endTime)) {
                    curPropertyValueList.add("");
                } else {
                    String endTimeDateStr = this.convertTimestampStrToDateStr(endTime);
                    curPropertyValueList.add(endTimeDateStr);                                    
                }
                String hostname = dqtl.getHostname();
                if(NullChecker.isNullOrEmpty(hostname) || hostname.equalsIgnoreCase("null")) {
                    curPropertyValueList.add("");
                } else {
                    curPropertyValueList.add(hostname);
                }
                String filename = dqtl.getFilename();
                if(NullChecker.isNullOrEmpty(filename) || filename.equalsIgnoreCase("null")) {
                    curPropertyValueList.add("");
                } else {
                    curPropertyValueList.add(filename);
                }
                String fileType = dqtl.getFiletype();
                if(NullChecker.isNullOrEmpty(fileType) || fileType.equalsIgnoreCase("null")) {
                     curPropertyValueList.add("");
                } else {
                     curPropertyValueList.add(fileType);
                }
                double filesize = dqtl.getFilesize();
                if(NullChecker.isNullOrEmpty(filesize)) {
                    curPropertyValueList.add("");
                } else {
                    curPropertyValueList.add(new Double(filesize).toString());
                }
                String destination = dqtl.getDestination();
                if(NullChecker.isNullOrEmpty(destination) || destination.equalsIgnoreCase("null")) {
                    curPropertyValueList.add("");
                } else {
                    curPropertyValueList.add(destination);
                }
                String status = dqtl.getStatus();
                if(NullChecker.isNullOrEmpty(status) || status.equalsIgnoreCase("null")) {
                    curPropertyValueList.add("");
                } else {
                    curPropertyValueList.add(status);
                }
                String localDocId = dqtl.getLocalDocId();
                if(NullChecker.isNullOrEmpty(localDocId) || localDocId.equalsIgnoreCase("null")) {
                    curPropertyValueList.add("");
                } else {
                    curPropertyValueList.add(localDocId);
                }
                String remoteDocId = dqtl.getRemoteDocId();
                if(NullChecker.isNullOrEmpty(remoteDocId) || remoteDocId.equalsIgnoreCase("null")) {
                    curPropertyValueList.add("");
                } else {
                    curPropertyValueList.add(remoteDocId);
                }
                String error = dqtl.getError();
                if(NullChecker.isNullOrEmpty(error) || error.equalsIgnoreCase("null")) {
                    curPropertyValueList.add(""); 
                } else {
                    curPropertyValueList.add(error); 
                }
                // put this list as an item in the big list
                dataList.add(curPropertyValueList);
            }
            report.setData(dataList);
        }
        //System.out.println("report toString():"+report);
        // Transform from Object to JSON String
        if(!NullChecker.isNullOrEmpty(report)) {
            Gson gson = new GsonBuilder().setPrettyPrinting().create();
            transformedJsonString = gson.toJson(report); 
        }
        LOG.debug("Returning transformedJsonString: "+transformedJsonString);
        return transformedJsonString;
    }   
    
    public List<Map<String, Object>> transformDataIntoExcelFormat(List<DqTransmissionLog> data) {
        List<Map<String, Object>> results = new ArrayList<Map<String, Object>>();
        for(DqTransmissionLog log: data) {
            Map<String, Object> map = new HashMap<String, Object>();
            map.put("dqTransmissionId", String.valueOf(log.getDqTransmissionId()));
            map.put("batchId", log.getBatchId());
            map.put("eventType", log.getEventType());
            map.put("startTime", log.getStartTime());
            map.put("startTimeFormatted",
                            this.convertTimestampStrToDateStr(log.getStartTime()));
            map.put("endTime", log.getEndTime());
            map.put("endTimeFormatted",
                            this.convertTimestampStrToDateStr(log.getEndTime()));            
            map.put("hostName", log.getHostname());
            map.put("fileName", log.getFilename());
            map.put("fileType", log.getFiletype());
            map.put("fileSize", String.valueOf(log.getFilesize()));
            map.put("destination", log.getDestination());
            map.put("status", log.getStatus());
            map.put("localDocId", log.getLocalDocId());
            map.put("remoteDocId", log.getRemoteDocId());
            map.put("error", log.getError());
            results.add(map);
        }
        return results;
    }
    
    public Map<String, Object> mapSessionAttributes(HttpServletRequest request)  throws ServletException,
    IOException {
        Map<String, Object> attributes = new HashMap<String, Object>();
        
        //attributes.put("userId", request.getParameter("userId"));
        attributes.put("startDate", ReportHelper.getStartDate(getParamWithoutHPP(request,"startDate")));
        attributes.put("endDate", ReportHelper.getEndDate(getParamWithoutHPP(request,"endDate")));
        attributes.put("eventType", getParamWithoutHPP(request,"eventType"));
        attributes.put("status", getParamWithoutHPP(request,"status"));
        attributes.put("filename", getParamWithoutHPP(request,"filename"));  

        return attributes;
    }    
    
	private String getParamWithoutHPP(HttpServletRequest request, String paramName) throws ServletException
	{
		String[] paramValues = request.getParameterValues(paramName);
		if(paramValues ==null || paramValues.length==0)
		{
			return StringUtils.EMPTY;
		}
		else if(paramValues.length > 1)
		{
			throw new ServletException(" Multiple request values, HPP attack averted ");
		}
		return paramValues[0];
	}

    public String convertTimestampStrToDateStr(String timestampStr) {
        Long timestamp = Long.parseLong(timestampStr);
        Date date = new Date(timestamp);
        // "11-JUL-17 10.10.02.397000000 AM"
        DateFormat format = new SimpleDateFormat("MM/dd/yyyy HH:mm:ss.S a");
        return format.format(date);        
    }    
    
    public class QueryResponseObj {
        
        private String tableName;
        private int resultCount;
        private List<DqTransmissionLog> data;

        public String getTableName() {
            return tableName;
        }

        public void setTableName(String tableName) {
            this.tableName = tableName;
        }

        public int getResultCount() {
            return resultCount;
        }

        public void setResultCount(int resultCount) {
            this.resultCount = resultCount;
        }

        public List<DqTransmissionLog> getData() {
            return data;
        }

        public void setData(List<DqTransmissionLog> data) {
            this.data = data;
        }
        
    }     
    
    public class DqTransmissionLogDataTablesReport {
        
        private String tableName;
        private int recordsFiltered;
        private List<List<String>> data;   
        private String recordsTotal;

        public String getTableName() {
            return tableName;
        }

        public void setTableName(String tableName) {
            this.tableName = tableName;
        }

        public int getRecordsFiltered() {
            return recordsFiltered;
        }

        public void setRecordsFiltered(int recordsFiltered) {
            this.recordsFiltered = recordsFiltered;
        }

        public List<List<String>> getData() {
            return data;
        }

        public void setData(List<List<String>> data) {
            this.data = data;
        }
        
        public String getRecordsTotal() {
            return recordsTotal;
        }

        public void setRecordsTotal(String recordsTotal) {
            this.recordsTotal = recordsTotal;
        }
        
    }
    
}
