package gov.va.nvap.web.report;

import gov.va.nvap.audit.BulkDownloadDocs;
import gov.va.nvap.svc.consentmgmt.stub.data.Organization;
import gov.va.nvap.web.app.ResponseDispatcherHttpServlet;
import gov.va.nvap.web.helper.privacy.ConsentManagementHelper;
import gov.va.nvap.web.helper.report.ReportHelper;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Properties;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.io.FilenameUtils;
import javax.servlet.jsp.tagext.ValidationMessage;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.identityconnectors.common.security.GuardedString;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import weblogic.security.internal.SerializedSystemIni;
import weblogic.security.internal.encryption.ClearOrEncryptedService;
import weblogic.security.internal.encryption.EncryptionService;

/**
 * Servlet implementation class AdapterDateRangeReport
 * 
 * @author Sricharan Reddy
 */
public class AdapterDateRangeReport extends ResponseDispatcherHttpServlet {
	static private final Log LOG = LogFactory
			.getLog(AdapterDateRangeReport.class);
	//Begin Constant Declarations for Class
	private static final long   serialVersionUID = 1L;
	
	private static final int    BYTES_DOWNLOAD = 1024;
	
	private static final String MOCK_FLAG = "mock.bulk.download";
	
	private static final String REPORT_HELPER_CLASS= "reportHelper";
	
	private static final String CMS_HELPER_CLASS = "cmsHelper";
	
	private static final String INPUT_DATE_FORMAT = "MM/dd/yyyy";
	
	private static final String OUTPUT_DATE_FORMAT = "yyyy-MM-dd";
	
	private static final String START_DATE_PARAM = "startDate";
	
	private static final String END_DATE_PARAM ="endDate";
        
        private static final String SELECTED_ORGS_PARAM = "newRight";
	
	private static final String PATIENT_IDENTIFIER_PARAM = "patientIdentifier";
    private static final String SSN_PARAM = "ssn";
    private static final String LAST_NAME_PARAM = "lastName";
    private static final String FIRST_NAME_PARAM = "firstName";
	
	private static final String CONTENT_TYPE = "application/zip";
	
	private static final String RESPONSE_HEADER =  "attachment; filename=auditedDocsExport.zip"; 
    private static final String PROP_FILE_LOC_SYSPROP  = "vap.bulkDownload.config";
    private static final String PROP_FILE_LOC_DFTL_VAL = "config/vap/bulkDownload.properties";
    private static final String VAP_ID_PROP = "vap.identifier";
    
	// End of Constants Declaration

    //***
    // Set by init()
    //***
    private String vapId;
    private ClearOrEncryptedService encSvc;

    /**
     * Default constructor. 
     */
    public AdapterDateRangeReport()
    {
    	
    }

    @Override
    public void init() throws ServletException
    {
        super.init();
        try
        {
            vapId = loadVAPId();
        }
        catch(IOException ex)
        {
            throw new ServletException(
                "Unable to load configuration for bulk download.", ex);
        }
        
        // works off convention that the current working directory is the domain home
        String domainHome = new File("dummy").getAbsoluteFile().getParent();
        EncryptionService encryptSrv = SerializedSystemIni.getEncryptionService(domainHome);
        encSvc = new ClearOrEncryptedService(encryptSrv);
    }

    /**
     * Helper method to get the CmsHelper instance
     * @return instance of ConsentManagementHelper
     */
    public ConsentManagementHelper getCmsHelper()
    {
        final ConsentManagementHelper cmsHelper = this.getBean(CMS_HELPER_CLASS,ConsentManagementHelper.class);
        return cmsHelper;
    }
    
    /**
     *  Method to get an instance of bulk download docs
     * @return instance of BulkDownloadDocs
     */
    public BulkDownloadDocs getbulkDownloadDocs()
    {
    	String useMock =""; 
    	useMock = System.getProperty(MOCK_FLAG)!=null?System.getProperty(MOCK_FLAG):"";
    	String mockClass="";    
    	 if(useMock.equalsIgnoreCase("true"))
         {
    	      mockClass = "bulkDownloadDocs-mock";     		 
    	 }    	   
    	 else
    	 {
    	      mockClass = "bulkDownloadDocs";    		
    	 }
		final BulkDownloadDocs bulkDownloadBean = getBean(mockClass, BulkDownloadDocs.class);
    	return bulkDownloadBean;    	
    }
    
    /**
     * Method  for the Report Helper class
     * @return instance of ReportHelper
     */
    private ReportHelper getReportHelper()
    {
        return this.getBean(REPORT_HELPER_CLASS, ReportHelper.class);
    }

	/**
	 * Servlet logic for getting the dates and generating the output file
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
	{
		// Get relevant session variables
		String startDate= StringUtils.defaultIfEmpty(getParamWithoutHPP(request,START_DATE_PARAM),"");
		String endDate = StringUtils.defaultIfEmpty(getParamWithoutHPP(request,END_DATE_PARAM),"");
                String selOrgIndices = StringUtils.defaultIfEmpty(getParamWithoutHPP(request,SELECTED_ORGS_PARAM),"").trim();
		String patientIdentifier= StringUtils.defaultIfEmpty(getParamWithoutHPP(request,PATIENT_IDENTIFIER_PARAM),"");
                String ssn = StringUtils.defaultIfEmpty(getParamWithoutHPP(request,SSN_PARAM), "");
                String lastName = StringUtils.defaultIfEmpty(getParamWithoutHPP(request,LAST_NAME_PARAM), "");
                String firstName = StringUtils.defaultIfEmpty(getParamWithoutHPP(request,FIRST_NAME_PARAM), "");

                this.getReportHelper().getOrgLists(request,this.getCmsHelper());
                //Initialize page with Default values
		 if(startDate.equalsIgnoreCase("") || endDate.equalsIgnoreCase(""))
		 {
			 unspecified(request, response);
			 return;
		 }
		 
		 if(!startDate.equalsIgnoreCase("") && !endDate.equalsIgnoreCase(""))			 
		 {
            String startAudit      = formatDateForAdapter(startDate, false);
            // end date is exclusive, so add a day to have it interpreted as midnight on next day
            String endAudit        = formatDateForAdapter(endDate, true);
            ArrayList<String> orgIds = new ArrayList<String>();
            LOG.debug("Selected orgs param: " + selOrgIndices);
            if (!"".equals(selOrgIndices)) {
                ArrayList<Organization> activeOrgs = (ArrayList<Organization>) request.getAttribute("activeOrgs");
                ArrayList<Organization> inactiveOrgs = (ArrayList<Organization>) request.getAttribute("inactiveOrgs");
                for (String index : selOrgIndices.split(",")) {
                    if (index.startsWith("Z")) {
                        // inactive selection
                        Integer intIndex = getValidOrgIndexOrNull(index.substring(1), inactiveOrgs.size());
                        if (intIndex == null) {
                            // reload form and show validation error
                            ArrayList<ValidationMessage> errors = new ArrayList<>();
                            ValidationMessage msg = new ValidationMessage("", "Please complete all required fields.");
                            errors.add(msg);
                            request.setAttribute("formErrors", errors);
                            unspecified(request, response);
                            return;
                        } else {
                            orgIds.add(inactiveOrgs.get(intIndex).getOrgOid());
                        }
                    } else {
                        Integer intIndex = getValidOrgIndexOrNull(index, activeOrgs.size());
                        if (intIndex == null) {
                            // reload form and show validation error
                            ArrayList<ValidationMessage> errors = new ArrayList<>();
                            ValidationMessage msg = new ValidationMessage("", "Please complete all required fields.");
                            errors.add(msg);
                            request.setAttribute("formErrors", errors);
                            unspecified(request, response);
                            return;
                        } else {
                            orgIds.add(activeOrgs.get(Integer.parseInt(index)).getOrgOid());
                        }
                    }
                }
            }
            boolean firstOrg = true;
            String organizationIds = "";
            for (String orgId : orgIds) {
                if (firstOrg) {
                    firstOrg = false;
                    organizationIds = orgId;
                } else {
                    organizationIds += "," + orgId;
                }
            }
            LOG.debug(organizationIds);
            InputStream is;
            try
            {
                GuardedString vapIdClear = ReportDataProcessor.toGuardedString(encSvc.decrypt(vapId));
                is = getbulkDownloadDocs().downloadRetrievedDocs(ReportDataProcessor.decryptToString(vapIdClear), startAudit, endAudit, organizationIds, patientIdentifier, ssn, lastName, firstName); // Get input stream from BulkDownload Class
            }catch(Exception e)
            {
                throw new ServletException(
                    "An exception occurred while attempting to communicate with Adapter Bulk Document Download.", e);
            }

            BufferedOutputStream bos = null;
            try
            {
                response.setContentType(CONTENT_TYPE);
                response.setHeader("Content-Disposition",RESPONSE_HEADER);		    	
                int read;
                byte[] bytes = new byte[BYTES_DOWNLOAD];
                bos = new BufferedOutputStream(response.getOutputStream());                
                while((read = is.read(bytes))!= -1)
                {
                    bos.write(bytes, 0, read);		    		
                }
                bos.flush();
            }
            finally
            {
                IOUtils.closeQuietly(bos);
                IOUtils.closeQuietly(is);
            }
		 }
     }

        private Integer getValidOrgIndexOrNull(String value, int collectionSize) {
            if (StringUtils.isNumeric(value)) {
                int index = -1;
                try {
                    index = Integer.parseInt(value);    // what if > maxInt?
                } catch (NumberFormatException e) {
                    return null;
                }
                if (index < 0 || index >= collectionSize) {
                    return null;
                } else {
                    return index;
                }
            } else {
                return null;
            }
        }
        
	/**
	 * 
	 * Method  to retrieve Http Params without Http Pollution 
	 * @param  request
	 * @param  paramName
	 * @return the single value of the request Param
	 * @throws ServletException
	 * 
	 */
	private String getParamWithoutHPP(HttpServletRequest request, String paramName) throws ServletException
	{
		String[] paramValues = request.getParameterValues(paramName);
		if(paramValues ==null || paramValues.length==0)
		{
			return StringUtils.EMPTY;
		}
		else if(paramValues.length > 1)
		{
			throw new ServletException(" Multiple request values, HPP attack averted ");
		}
		return paramValues[0];
	}

	// Method to Format the partner string for input to client function
	//private String formatStringforPartner(String organizationName)
	//{
   //     String partnerId= organizationName.replace('%', ' '); // Remove % signs from Partner ID
   //     partnerId = partnerId.replaceAll("\\s+", ""); // Replace whitespace characters
	//	return partnerId;
	//}

	//Method to format dates for Webservice client
	private String formatDateForAdapter(String inputDate, boolean addDay) 
	{
		SimpleDateFormat sdf= new SimpleDateFormat(INPUT_DATE_FORMAT);
		SimpleDateFormat dt = new SimpleDateFormat(OUTPUT_DATE_FORMAT);
		Date date = null;
		try {
			date = sdf.parse(inputDate);
            if(addDay)
            {
                Calendar cal = Calendar.getInstance();
                cal.setTime(date);
                cal.add(Calendar.DAY_OF_MONTH, 1);
                date = cal.getTime();
            }
		} catch (ParseException e) {
            throw new RuntimeException("Exception in date format.", e);
		}
		String returnDate=dt.format(date);
		return returnDate!=null?returnDate:inputDate;
	}

	/**
	 * @see HttpServlet #doPost(HttpServletRequest request, HttpServletResponse response)
	 */
    @Override
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		doGet(request, response);
	}

	@Override
	protected void unspecified(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException
	{
            final HttpSession session = request.getSession(false);
            ReportHelper.setDefaultSearchDates(session);
	    this.forward(request, response, "show");		
	}
    
    private String loadVAPId() throws IOException
    {
        File file;
//  	  String config = System.getProperty(PROP_FILE_LOC_SYSPROP);
//  	  if (!ESAPI.validator().isValidFileName("upload", config, allowedExtensions, false)) {
//  			throw new ValidationUploadException("Upload only simple filenames with the following extensions " + allowedExtensions, "Upload failed isValidFileName check");
//
//  	   }
        if (System.getProperty(PROP_FILE_LOC_SYSPROP) != null)
        {
        	final String path = FilenameUtils.normalize(System.getProperty(PROP_FILE_LOC_SYSPROP));
            file = new File(path);
        }
        else
        {
            file = new File(PROP_FILE_LOC_DFTL_VAL);
        }

        InputStream stream = null;
        Properties props = new Properties();
        try
        {
            stream = new FileInputStream(file);
            props.load(stream);
        }
        finally
        {
            IOUtils.closeQuietly(stream);
        }
        return props.getProperty(VAP_ID_PROP);
    } 
}
