package gov.va.nvap.web.util.xls;

import gov.va.nvap.common.validation.Assert;
import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.web.dao.UserDocumentDAO;
import gov.va.nvap.web.user.UserHelper;
import gov.va.nvap.web.user.document.UserDocument;
import gov.va.nvap.web.util.date.DateUtil;

import java.io.IOException;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.json.JSONException;
import org.json.JSONObject;
import org.springframework.web.util.HtmlUtils;

/**
 *
 * @author Zachary Tubb
 */
public class ScheduledExports extends gov.va.nvap.web.app.ResponseDispatcherHttpServlet {
    
    private static final long serialVersionUID = 1L;

    private final SimpleDateFormat df = new SimpleDateFormat("yyyyMMdd_HHmmss");

    private String encodeToJSON(HashMap<String, Object> results, String draw) throws ParseException {
        try {
            StringWriter json = new StringWriter();
            JSONObject obj = new JSONObject();
            List<List<String>> data = new ArrayList<List<String>>(); //overarching "data"
            SimpleDateFormat dt = new SimpleDateFormat("yyyy-MM-dd kk:mm:ss.S");
            SimpleDateFormat finalDate = new SimpleDateFormat("MM/dd/yyyy hh:mm a");
            List<UserDocument> documents = (List<UserDocument>) results.get("results");
            Long count = (Long) results.get("count");

            for (UserDocument doc : documents) {
                List<String> dataItem = new ArrayList<String>();
                
                String status = doc.getStatus();
                if ("In Progress".equalsIgnoreCase(status)) {
                    if (doc.getCreationDate() != null && DateUtil.addTime(doc.getCreationDate(), Calendar.MINUTE, 20).before(new Date())) {
                        status = "FAILED";
                    }
                }
                
                dataItem.add(doc.getTitle());
                dataItem.add(doc.getFormat());
                Date date = dt.parse(doc.getCreationDate().toString());
                dataItem.add(finalDate.format(date));
                dataItem.add(status);
                if (doc.getStatus() != null && doc.getStatus().equals("COMPLETED")) {
                    dataItem.add("<a href='ScheduledExports.do_sec?getExport=true&docId=" + doc.getDocumentId().toString()
                        + "'><button title='Download' class='btn btn-primary btn-xs'><span class='fa fa-download pad-right-5' aria-hidden='true'></span>Download</button></a>");
                } else {
                    dataItem.add("&nbsp;"); //add in a non-breaking space if it's not completed yet so the view doesn't show "null"
                }
                data.add(dataItem);
            }
            //Add in required Datables values
            obj.put("draw", draw);
            obj.put("recordsFiltered", count);
            obj.put("recordsTotal", count);
            obj.put("data", data);

            obj.write(json);

            return json.toString();
        } catch (JSONException ex) {
            Logger.getLogger(ScheduledExports.class.getName()).log(Level.SEVERE, null, ex);
        }
        return "";
    }

    private UserDocumentDAO getUserDocumentDAO() {
        return this.getBean("UserDocumentDAO", UserDocumentDAO.class);
    }

    //Method to retreive scheduled export from the database for download.
    public void getExport(HttpServletRequest request, HttpServletResponse response) throws ServletException {
        Long docId = Long.parseLong(request.getParameter("docId"));
        if (!NullChecker.isNullOrEmpty(docId)) {
            //Get the document from using the DAO.
            UserDocument document = this.getUserDocumentDAO().findUserDocument(docId);
            String date = df.format(new Date());

            if (!NullChecker.isNullOrEmpty(document)) {
                //Convert to bytes for output stream.
                byte[] wbBytes = (byte[]) document.getContent();
                String file = "";
                if(document.getTitle() != null)
                {
                    file = document.getTitle()+ "_" + date;
                    file = file.replaceAll("[\n|\r|\t]", "");
                    file = file.replaceAll(" ", "_");
                    String regex = "[\\_\\-\\.a-zA-Z0-9]+";
                    if( !file.matches(regex) 
                    		|| file.toLowerCase().contains("script")
                    		|| file.toLowerCase().contains("alert")){
                    	Logger.getLogger(ScheduledExports.class.getName()).log(Level.SEVERE, file +" is an invalid export file name ");
                    }
                    
                }

                //Set response attributes.
                switch (document.getFormat()) {
                    case "Excel":
                        response.setContentType("application/vnd.ms-excel");
                        response.setHeader("Content-Disposition",
                                "attachment; filename=\"" + file + ".xls" + "\"");
                        response.setHeader("Cache-Control", "must-revalidate");
                        break;
                    case "CSV":
                        response.setContentType("text/csv");
                        response.setHeader("Content-Disposition",
                                "attachment; filename=\"" + file + ".csv" + "\"");
                        response.setHeader("Cache-Control", "must-revalidate");
                        break;
                }

                try {
                    //Write to output stream.
                    final ServletOutputStream os = response.getOutputStream();
                    Assert.assertNotEmpty(os, "Output Stream cannot be null!");
                    os.write(wbBytes);
                    try {
                        os.close();
                    }
                    catch (final IOException ex) {
                    }
                } catch (final IOException ex) {
                    throw new ServletException(ex);
                }
            }
        }
    }

    public static byte[] getFormatedString(byte[] string) throws UnsupportedEncodingException{

        String str=new String(string,"UTF-8"); 
        str=HtmlUtils.htmlEscape(str);
        return str.getBytes("UTF-8");

    }
    public void getUserExports(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException, ParseException {
        //Get paramaters from Datables request
        final HttpSession session = request.getSession(false);
        final String maxRows = request.getParameter("length");
        final String startRow = request.getParameter("start");
        final String draw = request.getParameter("draw");
        final String sortBy = request.getParameter("order[0][column]");
        final String sortOrder = request.getParameter("order[0][dir]");

        //Get the exports with the parameters from Datables
        HashMap<String, Object> hm = this.getUserDocumentDAO().findByUserId(UserHelper.getUserName(request), sortBy, sortOrder, Integer.parseInt(maxRows), Integer.parseInt(startRow));

        session.setAttribute("results", encodeToJSON(hm, draw));

        this.forward(request, response, "resultsJSON");
    }

    @Override
    protected void unspecified(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        this.forward(request, response, "show");
    }

}

