<%-- 
    Document   : monthlyReceivedDocumentsReport
    Created on : Nov 4, 2016, 8:18:47 AM
    Author     : Johann Sonnenberg
--%>

<%@taglib tagdir="/WEB-INF/tags/formatting" prefix="f" %>
<%@taglib tagdir="/WEB-INF/tags/date" prefix="d" %>
<%@taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c" %>

<f:mainTemplate headerText="Monthly Received Documents Report">
    <script language="JavaScript">
        $(document).ready(function () { 
            //initial data table setup and call for the first recordset
            VAP.common.setProcessingForDataTable('tblReport', 'divReportPanel');
            
            VAP.common.reportTable = $('#tblReport').DataTable({
                deferLoading: 0,
                destroy: true,
                searching: false,
                processing: true,
                serverSide: true,
                paging: false,
                info: false,
                order: [[0, 'desc']],
                columnDefs: [
                    {className: "text-right", targets: [2]}
                ],
                "footerCallback": function ( row, data, start, end, display ) {
                    var api = this.api(), data;

                    // Update footer
                    $(api.column(2).footer() ).html(
                        api.column(2, {page:'current'}).data().sum()
                    );
                },
                ajax: { 
                    url: "MonthlyReceivedDocumentsReport.do_sec?doSearch=true&startYearMonth=" + $('#startYearMonth').val(),
                    dataSrc: function(json){
                        $('.dataTables_empty').remove();
                        return json.data;
                    },
                    error: function(o){
                        VAP.common.handleErrorsInDataTable('tblReport', o, 3, 'divReportPanel');
                    }
                }, 
                lengthMenu: [ 10, 25, 50, 100, 250, 500 ],
                pageLength: 25, 
                dom: 'lipt<"pad-bottom-20">ipr'
            });

            VAP.common.reportTable.on('draw', function () {
                VAP.common.addAriaToDataTable('tblReport', VAP.common.reportTable);
            });
            
            VAP.common.constructUrl = function(methodName) {
                var url = 'MonthlyReceivedDocumentsReport.do_sec?' + methodName + '=true&startYearMonth=' + $('#startYearMonth').val();

                if (methodName === 'exportToCsv' || methodName === 'exportToExcel') {
                    url = url + '&totalRows=' + VAP.common.reportTable.page.info().recordsTotal;
                }
                
                return url;
            }
            
            $('#btnSearch').on('click', function () {
                $('#tblReport .dataTables_empty').html(''); //blank out the empty table message if there is one
                $('#results').show();
                VAP.common.reportTable.ajax.url(VAP.common.constructUrl('doSearch')).load();
            });
            
            $('#results').hide();
        });
    </script>
    
    <div class="row">
        <div class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
            <div class="panel panel-default">
                <div class="panel-heading">
                    <div class="panel-title">
                        <a href="#" title="Close Search Criteria" aria-label="Close Search Criteria" class="vap-panel-hide-show"><i class="fa fa-minus" aria-hidden="true" data-panel-body="monthlyReceivedDocumentsReportSearchPanel"></i></a> Search Criteria
                    </div>
                </div>    
                <div class="panel-body" id="monthlyReceivedDocumentsReportSearchPanel">
                    <div id="errorMessage" class="alert alert-warning" style="display: none;" role="alert"></div>
                    <form name="searchForm" id="searchForm" class="form-horizontal">
                        <div class="form-group">
                            <label for="startYearMonth" class="col-xs-2 col-sm-2 col-md-2 col-lg-2 control-label">Start Month/Year</label>
                            <div class="col-xs-2 col-sm-2 col-md-2 col-lg-2">
                                <d:monthAndYearDropdown fieldId="startYearMonth" fieldName="startYearMonth" cssClass="form-control" enableMultiple="false" startYear="2010" startMonth="January" descriptor="${descriptors['monthlyReceivedDocumentReportStartMonthYearField']}" fieldTitle="Start Month and Year" />
                            </div>
                        </div>
                        <div class="form-group">
                            <div class="col-xs-4 col-sm-4 col-md-4 col-lg-4 text-right">
                                <input name="btnSearch" id="btnSearch" title="Search" class="btn btn-primary" type="button" value="Search" />
                            </div>
                        </div>
                    </form>
                </div>
            </div>
        </div>
    </div>
    <div><div aria-live="assertive" role="alert" id="reportTable_info"></div></div>
    <div id="results">
        <div class="row">
            <div id="divReportContent" class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
                <div class="panel panel-default" id="divReportPanel">
                    <div class="panel-heading">
                        <div class="panel-title">
                            <div class="pull-left">
                                <a href="#" title="Close Search Results" aria-label="Close Search Results" class="vap-panel-hide-show"><i class="fa fa-minus" aria-hidden="true" data-panel-body="monthlyReceivedDocumentsReportResultsPanel"></i></a> Search Results
                            </div>
                            <div class="pull-right">
                                <a href="#" class="directDownload" data-method="exportToExcel" title="Export to Excel" class="pad-right-10"><i class="fa fa-download" aria-hidden="true"></i> Export to Excel</a>
                            </div>
                            <div class="clearfix"></div>
                        </div>
                    </div>    
                    <div class="panel-body" id="monthlyReceivedDocumentsReportResultsPanel">
                        <table id="tblReport" class="table table-striped" width="100%" cellspacing="0" role="presentation">
                            <thead>
                                <tr role="row">
                                    <th role="columnheader" class="text-center"><span data-tooltip="<c:out value="${descriptors['monthlyReceivedDocumentReportRemoteOrganizationColumn']}" />">Remote Organization</span></th>
                                    <th role="columnheader" class="text-center"><span data-tooltip="<c:out value="${descriptors['monthlyReceivedDocumentReportUserColumn']}" />">User</span></th>
                                    <th role="columnheader" class="text-center"><span data-tooltip="<c:out value="${descriptors['monthlyReceivedDocumentReportCountColumn']}" />" data-tooltip-position="left">Count</span></th>
                                </tr>
                            </thead>
                            <tfoot>
                                <tr role="row">
                                    <%--Filled in by datatables, total--%>
                                    <td role="gridcell" class="text-left bold">Total:</td>
                                    <td role="gridcell"></td>
                                    <td role="gridcell" class="text-right bold"></td>
                                </tr>
                            </tfoot>
                        </table>
                        <div aria-live="polite" role="status" id="tblReport_info" class="dataTables_info hidden">Shows monthly received documents summary data</div><%-- because we don't have the "Showing x of x" turned on for this table, we need this here to support aria --%>        
                    </div>
                </div>
            </div>
        </div>
    </div>
</f:mainTemplate>