<%--
    Document   : Patient Discovery Report
    Created By : David Vasquez / Robert Snelling
    Modified By: Asha Amritraj
    Edited By: Zachary Tubb
--%>

<%@taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c"%>
<%@taglib tagdir="/WEB-INF/tags/cpp" prefix="cpp"%>
<%@taglib tagdir="/WEB-INF/tags/date" prefix="date"%>
<%@taglib tagdir="/WEB-INF/tags/formatting" prefix="f" %>
<%@page import="java.util.*"%>

<f:mainTemplate headerText="Patient Discovery Audit Report">
    <script type="text/javascript">
        validateSSNsBeforeExport = true; //setting the global variable to true so the ssns are validated prior to export

        function validateSsns() {
            var retVal = false;
            var sansWhiteSpace = $('#ssn').val().replace(/ /g, '');

            if (sansWhiteSpace === null) {
                $('#errorMessage').hide();
                retVal = true;
            } else {
                if (/^\d{9}(,\d{9})*$/.test(sansWhiteSpace) || sansWhiteSpace === '') {
                    $('#ssn').val(sansWhiteSpace);
                    $('#errorMessage').hide();
                    retVal = true;
                } else {
                    $('#errorMessage').html('Invalid SSN');
                    $('#errorMessage').show();
                }
            }
            return retVal;
        };
        
        VAP.common.constructUrl = function (methodName) {
            var url = 'PatientDiscoveryReport.do_sec?' + methodName + '=true' +
                    '&ssn=' + $('#ssn').val() +
                    '&lastName=' + $('#lastName').val() +
                    '&firstName=' + $('#firstName').val() +
                    '&userId=' + $('#userId').val() +
                    '&startDate=' + $('#startDate').val() +
                    '&startDateHour=' + $('#startDateHour').val() +
                    '&startDateMinute=' + $('#startDateMinute').val() + 
                    '&startDateMeridian=' + $('#startDateMeridian').val() +
                    '&endDate=' + $('#endDate').val() +
                    '&endDateHour=' + $('#endDateHour').val() +
                    '&endDateMinute=' + $('#endDateMinute').val() +
                    '&endDateMeridian=' + $('#endDateMeridian').val() +
                    //'&organization=' + $('#organization').val() +
                    //'&organizationName=' + $('#organizationName').val() +
                    '&sendingOrganization=' + $('#sendingOrganization').val() + 
                    '&sendingOrganizationName=' + $("#sendingOrganization option:selected").text().trim() +
                    '&receivingOrganization=' + $('#receivingOrganization').val() + 
                    '&receivingOrganizationName=' + $("#receivingOrganization option:selected").text().trim() +
                    '&mpiMatch=' + $('#mpiMatch').val() +
                    '&patientTypes=' + $('#patientTypes').val() +
                    '&sortBy=' + $('#reportTable').dataTable().fnSettings().aaSorting[0][0] +
                    '&sortOrder=' + $('#reportTable').dataTable().fnSettings().aaSorting[0][1];

            if (methodName === 'exportToCsv' || methodName === 'exportToExcel') {
                url = url + '&totalRows=' + VAP.common.reportTable.page.info().recordsTotal;
            }
            return url;
        };

        $(document).ready(function () {
            /*$("#startDate").datepicker({
                maxDate: 0,
                dateFormat: 'mm/dd/yy',
                onClose: function (selectedDate) {
                    $("#endDate").datepicker("option", "minDate", selectedDate);
                    $(this).focus();
                }
            });

            $("#endDate").datepicker({
                maxDate: 0,
                dateFormat: 'mm/dd/yy',
                onClose: function (selectedDate) {
                    $("#startDate").datepicker("option", "maxDate", selectedDate);
                    $(this).focus();
                }
            });*/
        
            $('#startDate').datepicker({
                theme: 'vap',
                inputFormat: 'MM/dd/yyyy',
                outputFormat: 'MM/dd/yyyy',
                gainFocusOnConstruction: false,
                onUpdate: function(curValue) {
                    $('#endDate').datepicker('min', curValue);
                }
            });
            
            $('#endDate').datepicker({
                theme: 'vap',
                inputFormat: 'MM/dd/yyyy',
                outputFormat: 'MM/dd/yyyy',
                gainFocusOnConstruction: false,
                onUpdate: function(curValue) {
                    $('#startDate').datepicker('max', curValue);
                }
            });

            VAP.common.setProcessingForDataTable('reportTable', 'divReportPanel');

            VAP.common.reportTable = $('#reportTable').DataTable({
                deferLoading: 0,
                ajax: {
                    url: VAP.common.constructUrl("doSearch"),
                    dataSrc: function (json) {
                        //Set summary information at bottom of page with callback data
                        $('#realPatientMessages').html(json.realPatientMessages);
                        $('#realPatientFails').html(json.realPatientFails);
                        $('#realPatientMatches').html(json.realPatientMatches);
                        $('#testPatientMessages').html(json.testPatientMessages);
                        $('#testPatientFails').html(json.testPatientFails);
                        $('#testPatientMatches').html(json.testPatientMatches);
                        $('#uniqueRealPatients').html(json.uniqueRealPatients);
                        $('#uniqueTestPatients').html(json.uniqueTestPatients);

                        configurePatientInformationDivs();

                        $('.dataTables_empty').remove();

                        //make sure to return main data
                        return json.data;
                    },
                    error: function (o) {
                        VAP.common.handleErrorsInDataTable('reportTable', o, 11, 'divReportPanel');
                    }
                },
                destroy: true,
                serverSide: true,
                processing: true,
                searching: false,
                paging: true,
                info: true,
                dom: "<'row'<'col-sm-12'l>>" +
                        "<'row'<'col-sm-6'i><'col-sm-6'p>>" +
                        "<'row'<'col-sm-12't<'pad-bottom-20'>r>>" +
                        "<'row'<'col-sm-5'i><'col-sm-7'p>>",
                lengthMenu: [10, 25, 50, 100, 250, 500],
                pageLength: 25,
                order: [[0, "desc"]],
                columnDefs: [
                    {orderable: false, targets: [9, 10]},
                    {
                        targets: 1,
                        data: null,
                        render: function (data, type, full, meta) {
                            return '<a href="#/" onclick="linkToPatient(\'' + data[11] + '\', \'' + data[2] + '\', \'' + data[3] + '\');">' + data[1] + '</a>';
                        }
                    },
                    {className: "vap-break-word vap-col-min-400", targets: 10},
                    {className: "vap-break-word vap-col-min-80", targets: [2, 8]},
                    {className: "vap-col-min-80", targets: 1},
                    {className: "vap-col-min-130", targets: 0}
                ]
            });

            function configurePatientInformationDivs() {
                if ($('#patientTypes').val() === "1") {
                    $('#realPatientInfo').show();
                    $('#testPatientInfo').hide();
                } else if ($('#patientTypes').val() === "2") {
                    $('#realPatientInfo').hide();
                    $('#testPatientInfo').show();
                } else {
                    $('#realPatientInfo').show();
                    $('#testPatientInfo').show();
                }
            }

            // DataTables overrides
            VAP.common.reportTable.on('draw', function () {
                VAP.common.addAriaToDataTable('reportTable', VAP.common.reportTable);
                VAP.common.addSrollingToDataTable('reportTable');
            });

            $('#searchButton').on('click', function () {
                $('#reportTable .dataTables_empty').html(''); //blank out the empty table message if there is one
                
                // Validate data before submitting to the server.
                if (validateSsns()) {
                    var fullStartDate = $('#startDate').val() + ' ' + $('#startDateHour').val() + ':' + $('#startDateMinute').val() + ' ' + $('#startDateMeridian').val();
                    var fullEndDate = $('#endDate').val() + ' ' + $('#endDateHour').val() + ':' + $('#endDateMinute').val() + ' ' + $('#endDateMeridian').val();
                    
                    var errorMessage = VAP.common.validateStartAndEndDateWithTime(fullStartDate, fullEndDate, true); //future dates are enabled
                    
                    var validationRslt = $('#patientDiscoveryReportForm').validator('validate');
                    if (validationRslt.has('.has-error').length)
                    {
                        errorMessage = 'Both sending and receiving organizations cannot be selected. Please choose only one.';
                    }
                    
                    if (errorMessage != '')
                    {
                        //we have an error so show the error, and then don't send the ajax request
                        $('#errorMessage').html(errorMessage);
                        $('#errorMessage').show();
                        
                        return false;
                    } else {
                        //no error, let's make the call to populate the report
                        $('#results').show();
                        VAP.common.reportTable.ajax.url(VAP.common.constructUrl('doSearch')).load();
                    }
                }
            });
            
            $('#patientDiscoveryReportForm').validator({
                custom: {
                    areBothOrgsSelected: function($el) { 
                        return ($('#sendingOrganization').val().length && $('#receivingOrganization').val().length); //!moment($el.val(), 'MM/DD/YYYY', true).isValid();
                    }
                }
            });
            
            $('#startDate,#endDate,#ssn,#startDateHour,#startDateMinute,#startDateMeridian,#endDateHour,#endDateMinute,#endDateMeridian').on('change', function () {
                //if either date or time changes or ssn, hide the error message if its not already hidden
                $('#errorMessage').hide();
            });
            
            $('#results').hide();
        });
    </script>

    <div class="row">
        <div class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
            <div class="panel panel-default">
                <div class="panel-heading">
                    <div class="panel-title">
                        <a href="#" title="Close Search Criteria" aria-label="Close Search Criteria" class="vap-panel-hide-show"><i class="fa fa-minus" aria-hidden="true" data-panel-body="myReportSearchPanel"></i></a> Search Criteria
                    </div> 
                </div>   
                <div class="panel-body" id="myReportSearchPanel"> 
                    <div class="alert alert-info">
                        Fill in the fields below to narrow your search. All dates will be included if Start Date and End Date are left blank.
                    </div>
                    <form class="form-horizontal" name="patientDiscoveryReportForm" id="patientDiscoveryReportForm" action="PatientDiscoveryReport.do_sec" method="POST" onsubmit="return doSubmit(this)" data-toggle="validator" role="form">
                        <div id="errorMessage" class="alert alert-warning pad-15 margin-bottom-10" style="display: none;" role="alert"></div>
                        <div class="row">
                            <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                <div class="form-group">
                                    <label for="ssn" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">SSN(s):</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <input class="form-control" title="One or multiple SSNs, with no hyphens, separated by comma" id="ssn" name="ssn" type="text" size="30" onblur="validateSsns();" onfocus="$('#ssnError').hide();"
                                               data-tooltip="<c:out value="${descriptors['patientDiscoveryAuditReportSSNField']}"/>"/>
                                        <i><span style="font-weight:normal;">(Enter one or more SSN's, with no hyphens, separated by commas)</span></i> &nbsp;
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label for="lastName" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">Last Name</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <input class="form-control" title="Enter Last Name" id="lastName" name="lastName" type="text" maxlength="255" size="30"
                                               data-tooltip="<c:out value="${descriptors['lastNameField']}"/>"/>
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label for="firstName" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">First Name</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <input class="form-control" title="Enter First Name" id="firstName" name="firstName" type="text" maxlength="255" size="30"
                                               data-tooltip="<c:out value="${descriptors['firstNameField']}"/>"/>
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label for="startDate" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">Start Date</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6" data-tooltip="<c:out value="${descriptors['startDateEndDateField']}" />">
                                        <input type="text" name="startDate" id="startDate" class="form-control" value="<c:out value="${today}" />" />
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label id="startTimeLabel" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">Start Time</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6 form-inline" data-tooltip="<c:out value="${descriptors['startDateEndDateTimeField']}" />">
                                        <date:timePicker prefixId="startDate" prefixName="startDate" hourSelected="12" labelPrefix="Start Time" />
                                    </div>
                                </div>     
                                <div class="form-group">
                                    <label for="endDate" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">End Date</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6" data-tooltip="<c:out value="${descriptors['startDateEndDateField']}" />">
                                        <input type="text" name="endDate" id="endDate" class="form-control" value="<c:out value="${tomorrow}" />" />
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label id="endTimeLabel" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">End Time</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6 form-inline" data-tooltip="<c:out value="${descriptors['startDateEndDateTimeField']}" />">
                                        <date:timePicker prefixId="endDate" prefixName="endDate" hourSelected="12" labelPrefix="End Time" />
                                    </div>
                                </div>    
                            </div>
                            <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                <div class="form-group">
                                    <label for="userId" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">User ID</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <input class="form-control" title="Enter User ID" id="userId" name="userId" type="text" maxlength="255" size="30"
                                               data-tooltip="<c:out value="${descriptors['genericUserId']}"/>"/>
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label" for="mpiMatch">MPI Results</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <select class="form-control" id="mpiMatch" name="mpiMatch" data-tooltip="<c:out value="${descriptors['mpiResultsField']}"/>">
                                            <option value="ALL">ALL</option>
                                            <option value="MATCH FOUND">Match Found</option>
                                            <option value="MATCH FAILED">Match Failed</option>
                                        </select>
                                    </div>
                                </div>
                                <%-- <div class="form-group">
                                    <label class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label"  for="organization">Organization</label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <select class="form-control" id="organization" name="organization" onchange="document.getElementById('organizationName').value = this.options[this.selectedIndex].text;"
                                                data-tooltip="<c:out value="${descriptors['eHealthExchangeOrganizationField']}"/>">
                                            <option value="">ALL</option>
                                            <c:forEach var="r" items="${activeOrgs}">
                                                <option value="%<c:out value="${r.orgOid}"/>"><c:out value="${r.orgName}"/></option>
                                            </c:forEach>
                                            <c:forEach var="r" items="${inactiveOrgs}">
                                                <option value="%<c:out value="${r.orgOid}"/>">[INACTIVE] <c:out value="${r.orgName}"/></option>
                                            </c:forEach>
                                        </select>
                                    </div>
                                </div> --%>
                                <div class="form-group">
                                    <label for="sendingOrganization" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">
                                        Sending Organization
                                    </label> 
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <select data-tooltip="<c:out value="${descriptors['disclosuresReportSendingOrganizationField']}"/>" id="sendingOrganization" name="sendingOrganization" class="form-control" data-areBothOrgsSelected="areBothOrgsSelected">
                                            <option value="" selected>ALL</option>
                                            <c:forEach var="r" items="${activeOrgs}">
                                                <option value="%<c:out value="${r.orgOid}"/>">
                                                    <c:out value="${r.orgName}"/>
                                                </option>
                                            </c:forEach>
                                            <c:forEach var="r" items="${inactiveOrgs}">
                                                <option value="%<c:out value="${r.orgOid}"/>">[INACTIVE] <c:out value="${r.orgName}"/></option>
                                            </c:forEach>
                                        </select>
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label for="receivingOrganization" class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label">
                                        Receiving Organization
                                    </label> 
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <select data-tooltip="<c:out value="${descriptors['disclosuresReportReceivingOrganizationField']}"/>" id="receivingOrganization" name="receivingOrganization" class="form-control" data-areBothOrgsSelected="areBothOrgsSelected">
                                            <option value="" selected>ALL</option>
                                            <c:forEach var="r" items="${activeOrgs}">
                                                <option value="%<c:out value="${r.orgOid}"/>">
                                                    <c:out value="${r.orgName}"/>
                                                </option>
                                            </c:forEach>
                                            <c:forEach var="r" items="${inactiveOrgs}">
                                                <option value="%<c:out value="${r.orgOid}"/>">[INACTIVE] <c:out value="${r.orgName}"/></option>
                                            </c:forEach>
                                        </select>
                                    </div>
                                </div>
                                <div class="form-group">
                                    <label class="col-xs-4 col-sm-4 col-md-4 col-lg-4 control-label" for="patientTypes">Patient Types<span style="display:none;">"Choose whether you would like to filter by real patients only, test patients only, or both."</span></label>
                                    <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                                        <select class="form-control" id="patientTypes" name="patientTypes" data-tooltip="<c:out value="${descriptors['patientTypesField']}"/>">
                                            <option value="3">ALL</option>
                                            <option value="1" selected>Real Patients</option>
                                            <option value="2">Test Patients</option>
                                        </select>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </form>
                    <div class="row">
                        <div class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
                            <div class="form-group">
                                <div class="text-right">
                                    <input id="searchButton" name="search" title="Search" class="btn btn-primary" type="submit" value="Search" />
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </div>
    <div><div aria-live="assertive" role="alert" id="reportTable_info"></div></div>
    <div id="results">
        <div class="panel panel-default" id="divReportPanel">
            <div class="panel-heading">
                <div class="panel-title">
                    <div class="pull-left">
                        <a href="#" title="Close Search Results" aria-label="Close Search Results" class="vap-panel-hide-show"><i class="fa fa-minus" aria-hidden="true" data-panel-body="myResultsPanel"></i></a> Search Results
                    </div>
                    <div class="pull-right">
                        <a href="#" data-toggle="modal" data-target="#confirmExportModal" data-method="exportToExcel" title="Export to Excel" class="pad-right-10"><i class="fa fa-download" aria-hidden="true"></i> Export to Excel</a>
                        <a href="#" data-toggle="modal" data-target="#confirmExportModal" data-method="exportToCsv" title="Export to CSV" class="pad-right-10"><i class="fa fa-download" aria-hidden="true"></i> Export to CSV</a>
                    </div>
                    <div class="clearfix"></div><!-- break out of the floating divs (this is required or well, you'll see if you do) -->
                </div>
            </div>   
            <div class="panel-body" id="myResultsPanel">
                <%@include file="patientLinkForm.jsp"%>
                <table id="reportTable" class="table table-striped" width="100%" cellspacing="0" role="presentation">
                    <thead>
                        <tr role="row">
                            <th role="columnheader" class="text-center"><span id="dateReceivedHeader" data-tooltip="<c:out value="${descriptors['eventOccurredColumn']}" />">Date Received (CT)</span></th>
                            <th role="columnheader" class="text-center"><span id="ssnHeader" data-tooltip="<c:out value="${descriptors['ssnColumn']}" />">SSN</span></th>
                            <th role="columnheader" class="text-center"><span id="lastNameHeader" data-tooltip="<c:out value="${descriptors['lastNameColumn']}" />">Patient Last Name</span></th>
                            <th role="columnheader" class="text-center"><span id="firstNameHeader" data-tooltip="<c:out value="${descriptors['firstNameColumn']}" />">Patient First Name</span></th>
                            <th role="columnheader" class="text-center"><span id="middleNameHeader" data-tooltip="<c:out value="${descriptors['middleNameColumn']}" />">Patient Middle Name</span></th>
                            <th role="columnheader" class="text-center"><span id="senderHeader" data-tooltip="<c:out value="${descriptors['sendingOrganizationColumn']}" />">Sender</span></th>
                            <th role="columnheader" class="text-center"><span id="pouHeader" data-tooltip="<c:out value="${descriptors['purposeOfUseColumn']}" />">Purpose of Use</span></th>
                            <th role="columnheader" class="text-center"><span id="receiverHeader" data-tooltip="<c:out value="${descriptors['receivingOrganizationColumn']}" />">Receiver</span></th>
                            <th role="columnheader" class="text-center"><span id="messageHeader" data-tooltip="<c:out value="${descriptors['messageColumn']}" />">Message</span></th>
                            <th role="columnheader" class="text-center"><span id="explanationHeader" data-tooltip="<c:out value="${descriptors['explanationOfFailureColumn']}" />">Explanation of Failure</span></th>
                            <th role="columnheader" class="text-center"><span id="detailsHeader" data-tooltip="<c:out value="${descriptors['detailsColumn']}" />" data-tooltip-position="left">Details</span></th>
                        </tr>
                    </thead>
                </table>
                <div>
                    <%--This are filled in via datatables data callback--%>
                    <div id='realPatientInfo' style="display: none">
                        <p><b>Total Real Patient Messages: </b><span id="realPatientMessages"></span></p>
                        <p><b>Total Real Patient Fails: </b><span id="realPatientFails"></span></p>
                        <p><b>Total Real Patient Matches: </b><span id="realPatientMatches"></span></p>
                        <p><b>Total Unique Real Patients: </b><span id="uniqueRealPatients"></span></p>
                    </div>
                    <div id='testPatientInfo' style="display: none">
                        <p><b>Total Test Patient Messages: </b><span id="testPatientMessages"></span></p>
                        <p><b>Total Test Patient Fails: </b><span id="testPatientFails"></span></p>
                        <p><b>Total Test Patient Matches: </b><span id="testPatientMatches"></span></p>
                        <p><b>Total Unique Test Patients: </b><span id="uniqueTestPatients"></span></p>
                    </div>
                </div>
            </div>
        </div>
    </div>
</f:mainTemplate>